import argparse
import warnings
import os
from concurrent.futures import ThreadPoolExecutor, as_completed

from dotenv import load_dotenv
from pydantic import BaseModel

from goose_checker.chains import (approve_or_deny, base_checker,
                                  terraform_checker)
from goose_checker.diff import get_git_diffs
from goose_checker.goose_ascii import detected_ascii, goose_ascii
from goose_checker.models import (AzureGooseChecker, GooseChecker,
                                  GooseCheckerResponse, OpenAIGooseChecker)


class AppConfig(BaseModel):
    branch: str
    model: str
    provider: str

    class Config:
        env_file = ".env"
        env_file_encoding = "utf-8"


def run_checker(diff, goose_checker: GooseChecker):
    if diff.file_name.endswith(".tf"):
        return terraform_checker(diff=diff, goose_checker=goose_checker)
    else:
        return base_checker(diff=diff, goose_checker=goose_checker)


def display_issues(responses: list[GooseCheckerResponse]):
    for response in responses:
        if response.chain_of_thought:  # Checks if silly_mistakes is not empty
            print("\n")
            print(f"File Name: {response.file_name}")
            print(f"Feedback: {response.chain_of_thought}")
            print("\n")


def parse_args() -> AppConfig:
    parser = argparse.ArgumentParser(description="Checks if you goofed")
    parser.add_argument(
        "--branch", nargs="?", default=os.environ.get("GOOSE_CHECKER_BRANCH_NAME", "main"),
        help="The branch or commit to compare against"
    )
    parser.add_argument(
        "--model", nargs="?", default=os.environ.get("GOOSE_CHECKER_MODEL_NAME", "gpt-3.5-turbo"),
        help="LLM model to use, for Azure instances this is the deployment ID."
    )
    supported_providers = ["azure", "openai"]
    parser.add_argument(
        "--provider", nargs="?", choices=supported_providers,
        default=os.environ.get("GOOSE_CHECKER_PROVIDER", "openai"),
        help=f"The cloud provider to use, choices are: {supported_providers}"
    )

    args = parser.parse_args()
    return AppConfig(**vars(args))

def main(config: AppConfig):
    
    warnings.filterwarnings("ignore", message="Importing verbose from langchain root module is no longer supported")
    print(f"Checking for geese by comparing to {config.branch}")
    print(f"Using model: {config.model} provided by {config.provider}")

    if config.provider == "azure":
        goose_checker = AzureGooseChecker(
            with_respect_to=config.branch, deployment_name=config.model
        )
    elif config.provider == "openai":
        goose_checker = OpenAIGooseChecker(
            with_respect_to=config.branch, model=config.model
        )
    else:
        raise ValueError(f"Provider specified not supported: {config.provider}")

    diffs = get_git_diffs(with_respect_to=config.branch)
    
    print("Checking these files for geese-like behaviour...")
    for diff in diffs:
        print(f"{diff.file_name}")

    if len(diffs) == 0:
        print(
            f"Compared your current checked out branch to {config.branch} and found nothing to check"
        )
        return False

    all_responses = []
    with ThreadPoolExecutor(max_workers=4) as executor:
        future_to_diff = {
            executor.submit(run_checker, diff, goose_checker): diff for diff in diffs
        }
        for future in as_completed(future_to_diff):
            diff = future_to_diff[future]
            try:
                resp = future.result()
                all_responses.append(resp)
            except Exception as exc:
                print(f"Failed to analyze {diff.file_name} due to {exc}")

    approved, explanation = approve_or_deny(all_responses, goose_checker)

    if not approved:
        print(goose_ascii)
        print(detected_ascii)
        print(explanation)
        return approved
    else:
        print("Approved! No Geese detected.")

    return approved


def cli_main():
    config = parse_args()
    main(config)


if __name__ == "__main__":
    load_dotenv()
    config = AppConfig(
        branch=os.environ.get("GOOSE_CHECKER_BRANCH_NAME", "main"),
        model=os.environ.get("GOOSE_CHECKER_MODEL_NAME", "gpt-3.5-turbo"),
        provider=os.environ.get("GOOSE_CHECKER_PROVIDER", "openai"),
    )
    main(config)



import subprocess

from pydantic import BaseModel

# Intelligently analyze git diff


class GitDiff(BaseModel):
    file_name: str
    diff: str


def get_all_files_with_diff(with_respect_to: str) -> list:
    """Get all files with a diff compared to the given branch or commit."""
    result = subprocess.run(
        ["git", "diff", "--name-only", with_respect_to], capture_output=True, check=True
    )
    result_list = list(result.stdout.decode().split("\n"))

    # Remove any empty string
    result_list = [x for x in result_list if x]

    return result_list


def _get_diff_for_file(file_name: str, with_respect_to: str) -> str:
    """Get the diff for a specific file compared to the given branch or commit."""
    result = subprocess.run(
        ["git", "diff", with_respect_to, file_name], capture_output=True, check=True
    )
    return result.stdout.decode()


def get_git_diffs(with_respect_to: str) -> list[GitDiff]:
    """Get all files with a diff compared to the given branch or commit."""
    files_with_diff = get_all_files_with_diff(with_respect_to)
    diffs = []
    for file in files_with_diff:
        try:
            diff = _get_diff_for_file(file, with_respect_to)
        except subprocess.CalledProcessError:
            print(f"Failed to get diff for {file}")
            continue
        diffs.append(GitDiff(file_name=file, diff=diff))
    return diffs



from langchain.output_parsers import ResponseSchema

approve_or_deny_schema = ResponseSchema(
    name="approve_or_deny",
    description="A boolean True or False to approve or deny the changes. A True to approve, False to deny.",
)
improvements_aggregation = ResponseSchema(
    name="instructions_to_engineer",
    description="Feedback to the engineer based on the issues. explain clearly how to fix them. Use numbered steps if possible.",
)

cot_issues = ResponseSchema(
    name="chain_of_thought_on_issues",
    description="Full thought process on issues within the code",
)

list_issues = ResponseSchema(
    name="list_issues",
    description="A comma separated list of issues found in the code",
)



terraform_prompt = """
<< INSTRUCTIONS >>
You are an software engineer reviewing a pull request by analyzing the DIFF of a terraform file
Keep in mind the DIFF may not be complete so at the edges of the diff you may see mistakes that are not actually mistakes.

Focus on the following:
- Spelling Mistakes
- Spelling consistencies
- Silly Mistakes


<< CONTEXT >>
{context_subprompt}

<< DIFF >>
{diff_subprompt}

<< TASK >>

<< FORMAT INSTRUCTIONS >>
{format_instructions}
"""

base_prompt = """
<< INSTRUCTIONS >>
You are an software engineer reviewing a pull request by analyzing the DIFF
Keep in mind the DIFF may not be complete so at the edges of the diff you may see mistakes that are not actually mistakes.

Focus on the following:
- Spelling Mistakes
- Spelling consistencies
- Silly Mistakes

<< DIFF >>
{diff_subprompt}

<< TASK >>

<< FORMAT INSTRUCTIONS >>
{format_instructions}

"""


aggregation_prompt = """
<< INSTRUCTIONS >>

You are a manager determining the whether code should be accepted or denied.
You will be given a list of EVALUATIONS done by your teams engineers.
Your job is to APPROVE or DENY the code by looking at the review.

<< EVALUATIONS >>

{evaluations_subprompt}

<< FORMAT INSTRUCTIONS >>

{format_instructions}
"""
